"""Shared animation constants and display helpers for consistent visual design."""

from __future__ import annotations

WAVE_FRAMES = [
    "ᘚᘚᘚᘚ",
    "ᘛᘚᘚᘚ",
    "ᘛᘛᘚᘚ",
    "ᘛᘛᘛᘚ",
    "ᘛᘛᘛᘛ",
    "ᘚᘛᘛᘛ",
    "ᘚᘚᘛᘛ",
    "ᘚᘚᘚᘛ",
]


WAVE_INTERVAL_MS = 100
